@@ -1,23 +1,30 @@
 from PyQt6.QtCore import Qt, QObject, QEvent, QSize
-from PyQt6.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QLabel, QPushButton, QLineEdit, QComboBox, QDialog, QToolButton, QWidgetAction, QCheckBox, QTableWidget, QHeaderView, QApplication, QStyle, QStyledItemDelegate
+from PyQt6.QtWidgets import QWidget, QVBoxLayout, QHBoxLayout, QLabel, QPushButton, QLineEdit, QComboBox, QDialog, QToolButton, QWidgetAction, QCheckBox, QTableWidget, QHeaderView, QApplication, QStyle, QStyledItemDelegate, QScrollArea, QSizePolicy
 from PyQt6.QtGui import QIcon, QTextDocument
 import re
 import requests
 import importlib.resources
 
-main_dir = importlib.resources.files("gui_utilities")
-icons_dir = main_dir / "icons"
-tlds_path = main_dir / "tlds"
+main_directory = importlib.resources.files("gui_utilities")
+icons_directory = main_directory / "icons"
+tlds_path = main_directory / "tlds"
 
-def latex_to_html(expression):
-    html_expression = expression
-    html_expression = re.sub(r"\\frac\{(.*?)\}\{(.*?)\}", r"<span style='font-size:90%;'>\1⁄\2</span>", html_expression)
-    html_expression = re.sub(r"_\{([^}]+)\}", r"<sub>\1</sub>", html_expression)
-    html_expression = re.sub(r"\^\{([^}]+)\}", r"<sup>\1</sup>", html_expression)
-    html_expression = re.sub(r"_([a-zA-Z0-9]+)", r"<sub>\1</sub>", html_expression)
-    html_expression = re.sub(r"\^([a-zA-Z0-9]+)", r"<sup>\1</sup>", html_expression)
-    html_expression = html_expression.replace(" ", "&nbsp;")
-    return html_expression
+def _get_alignment_flag(aligment):
+    if not isinstance(aligment, str): return aligment
+    alignment_map = {
+        "left": Qt.AlignmentFlag.AlignLeft,
+        "right": Qt.AlignmentFlag.AlignRight,
+        "center": Qt.AlignmentFlag.AlignCenter,
+        "justify": Qt.AlignmentFlag.AlignJustify,
+        "top": Qt.AlignmentFlag.AlignTop,
+        "bottom": Qt.AlignmentFlag.AlignBottom,
+        "middle": Qt.AlignmentFlag.AlignVCenter
+    }
+    parts = aligment.lower().split()
+    if not parts: return None
+    final_alignment = Qt.AlignmentFlag(0)
+    for part in parts: final_alignment |= alignment_map.get(part, Qt.AlignmentFlag(0))
+    return final_alignment if final_alignment != Qt.AlignmentFlag(0) else None
 
 def create_window(title, background_color = "#1e1e1e", parent = None):
     window = QWidget(parent)
@@ -44,9 +51,18 @@ def create_menu(
     title_border_width = 0,
     title_border_color = "#ffffff",
     title_border_radius = 0,
-    math_expression = False,
-    title_alignment = Qt.AlignmentFlag.AlignCenter
+    scrollbar_background_color = "#1e1e1e",
+    scrollbar_handle_background_color = "#4a4a4a",
+    hover_scrollbar_handle_background_color = "#333333",
+    hover_scrollbar_handle_border_width = 3,
+    hover_scrollbar_handle_border_color = "#777777",
+    pressed_scrollbar_handle_background_color = "#333333",
+    pressed_scrollbar_handle_border_width = 3,
+    pressed_scrollbar_handle_border_color = "#0078d7",
+    title_alignment = "center",
+    math_expression = False
 ):
+    title_alignment = _get_alignment_flag(title_alignment)
     main_layout = QVBoxLayout()
     main_layout.setContentsMargins(25, 25, 25, 25)
     main_layout.addLayout(create_title(
@@ -63,8 +79,8 @@ def create_menu(
         border_width = title_border_width,
         border_color = title_border_color,
         border_radius = title_border_radius,
-        math_expression = math_expression,
-        alignment = title_alignment
+        alignment = title_alignment,
+        math_expression = math_expression
     ))
     body_layout = QHBoxLayout()
     main_layout.addLayout(body_layout)
@@ -93,8 +109,34 @@ def create_menu(
         menu_layout.addWidget(button)
     ui_instance.content_widget = QWidget()
     ui_instance.content_widget.setLayout(QVBoxLayout())
-    body_layout.addWidget(ui_instance.content_widget)
     ui_instance.content_widget.setStyleSheet("background-color: #333333;")
+    scroll_area = QScrollArea()
+    scroll_area.setWidgetResizable(True)
+    scroll_area.setWidget(ui_instance.content_widget)
+    style_sheet = f"""
+        QScrollArea {{
+            border: none;
+            background-color: transparent;
+        }}
+        QScrollBar:vertical {{
+            background-color: {scrollbar_background_color};
+            border: none;
+        }}
+        QScrollBar::handle:vertical {{
+            background-color: {scrollbar_handle_background_color};
+            border: none;
+        }}
+        QScrollBar::handle:vertical:hover {{
+            background-color: {hover_scrollbar_handle_background_color};
+            border: {hover_scrollbar_handle_border_width}px solid {hover_scrollbar_handle_border_color};
+        }}
+        QScrollBar::handle:vertical:pressed {{
+            background-color: {pressed_scrollbar_handle_background_color};
+            border: {pressed_scrollbar_handle_border_width}px solid {pressed_scrollbar_handle_border_color};
+        }}
+    """
+    scroll_area.setStyleSheet(style_sheet)
+    body_layout.addWidget(scroll_area)
     body_layout.setStretch(0, 1)
     body_layout.setStretch(1, 4)
     return main_layout
@@ -117,9 +159,10 @@ def create_header(
     title_border_width = 0,
     title_border_color = "#ffffff",
     title_border_radius = 0,
-    math_expression = False,
-    title_alignment = Qt.AlignmentFlag.AlignCenter
+    title_alignment = "center",
+    math_expression = False
 ):
+    title_alignment = _get_alignment_flag(title_alignment)
     main_layout = QVBoxLayout()
     main_layout.setContentsMargins(margin_left, margin_top, margin_right, margin_bottom)
     title_layout = create_title(
@@ -136,8 +179,8 @@ def create_header(
         border_width = title_border_width,
         border_color = title_border_color,
         border_radius = title_border_radius,
-        math_expression = math_expression,
-        alignment = title_alignment
+        alignment = title_alignment,
+        math_expression = math_expression
     )
     main_layout.addLayout(title_layout)
     main_layout.addStretch()
@@ -157,9 +200,10 @@ def create_title(
     border_width = 0,
     border_color = "#ffffff",
     border_radius = 0,
-    math_expression = False,
-    alignment = Qt.AlignmentFlag.AlignCenter
+    alignment = "center",
+    math_expression = False
 ):
+    alignment = _get_alignment_flag(alignment)
     title_layout = QHBoxLayout()
     title_layout.setContentsMargins(0, 0, 0, 25)
     title_label = create_label(
@@ -177,8 +221,8 @@ def create_title(
         border_width = border_width,
         border_color = border_color,
         border_radius = border_radius,
-        math_expression = math_expression,
-        alignment = alignment
+        alignment = alignment,
+        math_expression = math_expression
     )
     title_layout.addWidget(title_label)
     return title_layout
@@ -205,16 +249,19 @@ def create_label(
         disabled_background_color = "transparent",
         disabled_border_width = 0,
         disabled_border_color = "#4a4a4a",
+        alignment = "left",
+        maximum_width = None,
+        word_wrap = False,
         math_expression = False,
         transparent_for_mouse = False,
-        alignment = None,
-        word_wrap = False,
         parent = None
 ):
     if math_expression:
-        label = QLabel(latex_to_html(text), parent)
+        label = QLabel(format_html(text), parent)
         label.setTextFormat(Qt.TextFormat.RichText)
     else: label = QLabel(text, parent)
+    alignment = _get_alignment_flag(alignment)
+    if maximum_width is not None: label.setMaximumWidth(maximum_width)
     if transparent_for_mouse: label.setAttribute(Qt.WidgetAttribute.WA_TransparentForMouseEvents, True)
     if alignment: label.setAlignment(alignment)
     if word_wrap: label.setWordWrap(True)
@@ -275,6 +322,7 @@ def create_button(
     disabled_border_width = 2,
     disabled_border_color = "#4a4a4a",
     maximum_width = 360,
+    alignment = "center",
     math_expression = False,
     parent = None
 ):
@@ -294,8 +342,8 @@ def create_button(
         padding_right = padding_right,
         padding_bottom = padding_bottom,
         disabled_border_width = disabled_border_width,
+        alignment = alignment,
         transparent_for_mouse = True,
-        alignment = Qt.AlignmentFlag.AlignCenter,
         math_expression = math_expression
     )
     label.setWordWrap(True)
@@ -362,10 +410,10 @@ def create_text_box(
     disabled_background_color = "#2d2d2d",
     disabled_border_width = 2,
     disabled_border_color = "#4a4a4a",
-    show_text_icon_url = str(icons_dir / "show_text_icon.png"),
-    hide_text_icon_url = str(icons_dir / "hide_text_icon.png"),
-    focused_show_text_icon_url = str(icons_dir / "focused_show_text_icon.png"),
-    focused_hide_text_icon_url = str(icons_dir / "focused_hide_text_icon.png"),
+    show_text_icon_url = str(icons_directory / "show_text_icon.png"),
+    hide_text_icon_url = str(icons_directory / "hide_text_icon.png"),
+    focused_show_text_icon_url = str(icons_directory / "focused_show_text_icon.png"),
+    focused_hide_text_icon_url = str(icons_directory / "focused_hide_text_icon.png"),
     hide_text = False,
     math_expression = False,
     parent = None
@@ -440,7 +488,6 @@ def create_text_box(
         toggle_text_visibility_button.clicked.connect(toggle_visibility)
 
         class _IconFocusWatcher(QObject):
-
             def __init__(self, watched, on_focus_change):
                 super().__init__(watched)
                 self._watched = watched
@@ -479,7 +526,6 @@ def create_text_box(
         if hide_text: update_icon()
     
     class _PlaceholderFocusWatcher(QObject):
-
         def __init__(self, watched, on_focus_change):
             super().__init__(watched)
             self._watched = watched
@@ -588,7 +634,7 @@ def create_combo_box(
         )
         combo_box.setItemDelegate(delegate)
         combo_box.clear()
-        for item in items: combo_box.addItem(latex_to_html(item))
+        for item in items: combo_box.addItem(format_html(item))
         combo_box.view().setTextElideMode(Qt.TextElideMode.ElideNone)
     else:
         combo_box.setPlaceholderText(placeholder_text)
@@ -607,7 +653,6 @@ def create_combo_box(
             padding_bottom = padding_bottom,
             math_expression = True,
             transparent_for_mouse = True,
-            alignment = Qt.AlignmentFlag.AlignCenter,
             parent = combo_box
         )
         selected_item_label.setVisible(False)
@@ -617,8 +662,7 @@ def create_combo_box(
                 html_text = combo_box.itemText(index)
                 selected_item_label.setText(html_text)
                 selected_item_label.setVisible(True)
-            else:
-                selected_item_label.setVisible(False)
+            else: selected_item_label.setVisible(False)
 
         combo_box.currentIndexChanged.connect(update_selected_item_display)
     
@@ -790,12 +834,14 @@ def create_information_message_box(
     button_disabled_background_color = "#2d2d2d",
     button_disabled_border_width = 2,
     button_disabled_border_color = "#4a4a4a",
+    fixed_width = 360,
+    math_expression = False,
     parent = None
 ):
     message_box = QDialog(parent)
     message_box.setObjectName("message_box")
     message_box.setWindowFlags(Qt.WindowType.FramelessWindowHint)
-    message_box.setFixedWidth(360)
+    if fixed_width is not None: message_box.setFixedWidth(fixed_width)
     style_sheet = f"""
         QDialog#message_box {{
             background-color: {background_color};
@@ -812,7 +858,7 @@ def create_information_message_box(
         text = text,
         font_family = label_font_family,
         font_size = label_font_size,
-        font_color=  label_font_color,
+        font_color = label_font_color,
         background_color = "transparent",
         padding = label_padding,
         padding_left = label_padding_left,
@@ -822,10 +868,11 @@ def create_information_message_box(
         border_width = label_border_width,
         border_color = label_border_color,
         border_radius = label_border_radius,
-        alignment = Qt.AlignmentFlag.AlignCenter
+        word_wrap = True,
+        math_expression = math_expression
     )
     main_layout.addWidget(text_label)
-    text_label.setWordWrap(True)
+    text_label.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
     accept_button = create_button(
         text = button_text,
         font_family = button_font_family,
@@ -900,12 +947,14 @@ def create_confirmation_message_box(
     button_disabled_background_color = "#2d2d2d",
     button_disabled_border_width = 2,
     button_disabled_border_color = "#4a4a4a",
+    fixed_width = 360,
+    math_expression = False,
     parent = None
 ):
     confirm_message_box = QDialog(parent)
     confirm_message_box.setObjectName("confirm_message_box")
     confirm_message_box.setWindowFlags(Qt.WindowType.FramelessWindowHint)
-    confirm_message_box.setFixedWidth(360)
+    if fixed_width is not None: confirm_message_box.setFixedWidth(fixed_width)
     style_sheet = f"""
         QDialog#confirm_message_box {{
             background-color: {background_color};
@@ -932,10 +981,11 @@ def create_confirmation_message_box(
         border_width = label_border_width,
         border_color = label_border_color,
         border_radius = label_border_radius,
-        alignment = Qt.AlignmentFlag.AlignCenter
+        word_wrap = True,
+        math_expression = math_expression
     )
     main_layout.addWidget(text_label)
-    text_label.setWordWrap(True)
+    text_label.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
     buttons_layout = QHBoxLayout()
     main_layout.addLayout(buttons_layout)
     buttons_layout.setSpacing(button_spacing)
@@ -1009,6 +1059,7 @@ def create_list_table(
     item_font_color = "#ffffff",
     item_background_color = "#1e1e1e",
     selected_item_background_color = "#0078d7",
+    item_alignment = "center",
     header_font_family = "Segoe UI",
     header_font_size = 14,
     header_font_color = "#ffffff",
@@ -1104,10 +1155,11 @@ def create_list_table(
     """
     list_table.setStyleSheet(style_sheet)
     list_table.setRowCount(len(items_data))
+    alignment_flag = _get_alignment_flag(item_alignment)
     for row, item_data in enumerate(items_data):
         items = row_populator_function(item_data)
-        for item in items:
-            item.setTextAlignment(Qt.AlignmentFlag.AlignCenter)
+        if alignment_flag:
+            for item in items: item.setTextAlignment(alignment_flag)
         for column, item in enumerate(items):
             list_table.setItem(row, column, item)
     return list_table
@@ -1236,15 +1288,17 @@ def validate_string(string, suffix = "El", field = "campo"):
     return f"{suffix} {field} no puede dejarse {"vacío" if suffix == "El" else "vacía"}."
 
 def validate_integer(integer, suffix = "El", field = "campo"):
-    if not integer or not integer.strip(): return f"{suffix} {field} no puede dejarse {"vacío" if suffix == "El" else "vacía"}."
+    if not integer or not integer.strip():
+        return f"{suffix} {field} no puede dejarse {"vacío" if suffix == "El" else "vacía"}."
     pattern = re.compile(r"^\d+$")
     unformatted_integer = integer.replace(".", "")
     if pattern.match(unformatted_integer): return None
     return f"No ha ingresado {"un" if suffix == "El" else "una"} {field} {"válido" if suffix == "El" else "válida"}."
 
 def validate_float(decimal, suffix = "El", field = "campo"):
-    if not decimal or not decimal.strip(): return f"{suffix} {field} no puede dejarse {"vacío" if suffix == "El" else "vacía"}."
-    pattern = re.compile(r"^\d+(\.\d+)?$")
+    if not decimal or not decimal.strip():
+        return f"{suffix} {field} no puede dejarse {"vacío" if suffix == "El" else "vacía"}."
+    pattern = re.compile(r"^-?\d{1,3}(?:\.\d{3})*(?:,\d+)?$|^-?\d+(?:,\d+)?$")
     if pattern.match(decimal): return None
     return f"No ha ingresado {"un" if suffix == "El" else "una"} {field} {"válido" if suffix == "El" else "válida"}."
 
@@ -1255,12 +1309,13 @@ def validate_id(id_str):
     return "No ha ingresado un D.N.I. válido."
 
 def validate_cellphone_number(cellphone_number):
-    if not cellphone_number or not cellphone_number.strip(): return "El número telefónico no puede dejarse vacío."
+    if not cellphone_number or not cellphone_number.strip():
+        return "El número telefónico no puede dejarse vacío."
     clean_number = "".join(filter(str.isdigit, cellphone_number))
     if len(clean_number) == 10: return None
     return "No ha ingresado un número telefónico válido."
 
-tlds_list_path = main_dir / tlds_path / "tlds_list.txt"
+tlds_list_path = main_directory / tlds_path / "tlds_list.txt"
 
 def export_tlds(tlds_list):
     try:
@@ -1325,4 +1380,18 @@ def format_date(date):
     day = f"{date.day:02d}"
     month = f"{date.month:02d}"
     year = f"{date.year:,}".replace(",", ".")
-    return f"{day}/{month}/{year}"
\ No newline at end of file
+    return f"{day}/{month}/{year}"
+
+def format_html(expression):
+    html_expression = expression
+    html_expression = re.sub(r"\\frac\{(.*?)\}\{(.*?)\}", r"<span style='font-size:90%;'>\1⁄\2</span>", html_expression)
+    html_expression = re.sub(r"_\{([^}]+)\}", r"<sub>\1</sub>", html_expression)
+    html_expression = re.sub(r"\^\{([^}]+)\}", r"<sup>\1</sup>", html_expression)
+    html_expression = re.sub(r"_([a-zA-Z0-9]+)", r"<sub>\1</sub>", html_expression)
+    html_expression = re.sub(r"\^([a-zA-Z0-9]+)", r"<sup>\1</sup>", html_expression)
+    html_expression = html_expression.replace(" ", "&nbsp;")
+    return html_expression
+
+def convert_to_float(number): return float(number.replace(".", "").replace(",", "."))
+
+def define_equality_symbol(number, number_rounded): return "=" if number == number_rounded else "≅"
\ No newline at end of file
